﻿Type=Class
Version=2.8
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class: rpiconsole.bas
'
Sub Class_Globals
	Private ws As WebSocket
	Private btnabout As JQueryElement		'Alert to show info about this app
	Private lbldatetime As JQueryElement	'Label showing Hello World
	Private lblauthor As JQueryElement
	Private lblcommandrun As JQueryElement
	Private consoleoutput As JQueryElement
	Private tasettingscontent As JQueryElement
	'Class Map holding the settings
  	Public  SettingsMap As Map              
  	'Settings file which is read by the global settings map
  	Public  SettingsFile As String = "settings.txt"   
End Sub

Public Sub Initialize
	
End Sub

#Region WebSocket
' The user opens via browser <URL>:51042. The html file index.html is displayed.
Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	Log("WebSocket_Connected :: Start")
	ws = WebSocket1
	'Read the settings
	ReadSettings
	'Update DOM elements in the index.html
	Dim hostname As String = Main.joInlineJava.RunMethod("ExecuteCommand", Array("hostname -I"))
	lbldatetime.setHtml("IP Host / Client:" & hostname & " / " & ws.UpgradeRequest.RemoteAddress & _ 
						" at " & DateTime.Date(DateTime.Now) & "&nbsp;" & DateTime.Time(DateTime.Now))
	lblauthor.SetHTML(Main.CAPPVERSION & ", " & Main.CAPPCOPYRIGHT)
	'Set the start info for the result - this will be updated with the command being executed
	lblcommandrun.SetHtml("Result Command:")
	'Define and set the selectmenu items - these are taken from the settingsmap
	Dim sel As String
	For i = 0 To SettingsMap.Size - 1
		'The items are appended to the commandselectmenu
		'Value = Settingsmap Value, Text = Settingsmap Key
		If i = 0 Then 
			sel = "selected"
		Else 
			sel = ""
		End If
		Dim item As String = "$('#commandmenu').append(""<option " & sel & " value='" & _ 
							 WebUtils.EscapeHtml(SettingsMap.GetValueAt(i)) & "'>" & _ 
							 WebUtils.EscapeHtml(SettingsMap.GetKeyAt(i)) & "</option>"");"
		'Log(item)
		ws.Eval(item, Null)
	Next
	'jQM requires a refresh when selecting first value - below is a trick to do so
	ws.Eval("var myselect = $(""#commandmenu""); myselect[0].selectedIndex = 0; myselect.selectmenu(""refresh"");", Null)
	Log("WebSocket_Connected :: End")
End Sub

'Not used
Private Sub WebSocket_Disconnected
End Sub

#End Region

#Region Settings
'Load the settings in global SettingsMap from the settings.txt file stored in the DirApp folder.
Sub ReadSettings
	Log("ReadSettings :: Start")
	'Check if the settings file exists, else create a new file
	If File.Exists(File.DirApp, SettingsFile) Then
    	SettingsMap = File.ReadMap(File.DirApp, SettingsFile)
	Else
    'Create a new map
    	SettingsMap = CreateMap ("Version": Main.CAPPVERSION)
    	File.WriteMap(File.DirApp, SettingsFile, SettingsMap)
	End If
	SortMapKeys(SettingsMap, True)
	tasettingscontent.SetText(File.ReadString(File.DirApp, SettingsFile))
Log("ReadSettings :: End")
End Sub

#End Region

'NOT USED = <button id="btnabout" title="About">About</button>
'Handle click on the button with id btnAddNewButton - Note In the index.html all id's are in lowercase
Sub btnAbout_click(params As Map)
	Log("btnabout_click :: Start = " & params.Size)
	'In index.html a div with id btnabout is defined. This is used to display the about info.
	ws.Alert("About" & CRLF & Main.CAPPVERSION & CRLF & CRLF & "by Robert W.B. Linn, Pinneberg, Germany" & CRLF & "www.rwblinn.de")
	Log("btnnabout_click :: End")
End Sub

'Run a command. Uses the Inline Java code defined in module Main.
'if required add: If cmd <> "null" Then 
'Parameter: Command as string
Sub runcommand(cmd As String)
	If cmd.Trim.Length = 0 Then
		consoleoutput.SetText("Error" & CRLF & "No Command given to run.")
		Return
	End If
	Try
		'Run the command
		Dim result As String = Main.joInlineJava.RunMethod("ExecuteCommand", Array(cmd))
		'Show the result
		lblcommandrun.SetHtml("Result Command: <B>" & cmd & "</B>")
		consoleoutput.SetText(result) ' WebUtils.EscapeHtml(result))
		'This is a trick to autoexpand the textarea after putting content.
		ws.Eval("$(""#consoleoutput"").trigger('keyup');", Null)
	Catch
		consoleoutput.SetText("Error" & CRLF & LastException.Message)
		Log(LastException.Message)
	End Try
End Sub

'Run the command selected via selectmenu
Sub runcommandmenu_event(params As Map)
	Log("runcommandmenu_event :: Start = " & params.Size)
	If params.Size <> 1 Then
		'Show an error
		ws.Alert("Error: No command selected.")
		Return
	End If
	runcommand(params.GetValueAt(0))
	Log("runcommandmenu_event :: End")
End Sub

'Run the command entered direct
Sub runcommanddirect_event(params As Map)
	Log("runcommanddirect_event :: Start = " & params.Size)
	If params.Size <> 1 Then
		'Show an error
		ws.Alert("Error: No command entered.")
		Return
	End If
	runcommand(params.GetValueAt(0))
	Log("runcommanddirect_event :: End")
End Sub

#Region DialogSettings
'Dialog popupDialogSettings
'Handle click on the button with id btnSettingsSave - Note In the index.html all id's are in lowercase
Sub btnSettingsSave_event(params As Map)
	Log("btnSettingsSave :: Start = " & params.Size)
	'Log(params.GetValueAt(0))
	'Update the settings file using the parameter value
	'This is not used as not gatting actual value via tasettingscontent.GetText.Value
	File.WriteString(File.DirApp, SettingsFile, params.GetValueAt(0))
	'Show message settings saved
	ws.Alert("Information" & CRLF & CRLF & "Settings updated in folder" & CRLF & File.DirApp)
	Log("btnSettingsSave_click :: End")
End Sub

#End Region

#Region Utilities
'Sort a Map by Key - using a list
'Thanks to luke2012 - see:
'www.b4x.com/android/forum/threads/how-to-sort-map-by-value.15612/#post88664
Sub SortMapKeys (m As Map, SortAsc As Boolean)
	Private KeysList As List : KeysList.Initialize
   	Private m2 As Map:m2.Initialize
  
   	For i = 0 To m.Size - 1
    	Private key As String = m.GetKeyAt(i)
      	KeysList.Add(key)
   	Next
  
   	KeysList.Sort(SortAsc)
  
   	For i=0 To KeysList.Size - 1
       	Private key As String = KeysList.Get(i)
       	Private val As String = m.Get(key)
       	m2.Put(key, val)
   	Next
   	m.Clear
   	For Each m2Key As String In m2.Keys
   		m.Put(m2Key, m2.Get(m2Key))
	Next
End Sub

#End Region
